/**********************************************************************
 *
 * ptmxdocaction.cpp -- 
 * A component of the Greenstone digital library software
 * from the New Zealand Digital Library Project at the
 * University of Waikato, New Zealand.
 *
 * Copyright (C) 2002  Stefan Boddie
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 *********************************************************************/

#include "ptmxdocaction.h"
#include "OIDtools.h"
#include "querytools.h"

ptmxdocaction::ptmxdocaction () {
}

ptmxdocaction::~ptmxdocaction () {
}

void ptmxdocaction::define_internal_macros (displayclass &disp, cgiargsclass &args, 
					    recptprotolistclass *protos, ostream &logout) {

  documentaction::define_internal_macros(disp, args, protos, logout);

  // don't want header overridden for protemix collection
  disp.setmacro ("header", "document", "_Global:header_");

  // set _pagepdf_ metadata

  if (!args["d"].empty()) {
    text_t &collection = args["c"];
    if (collection.empty()) return;
    recptproto *collectproto = protos->getrecptproto (collection, logout);
    if (collectproto == NULL) return;
    
    FilterResponse_t response;
    text_tset metadata;
    metadata.insert("pdf");
    if (get_info (args["d"], collection, metadata, false, collectproto, response, logout)) {
      text_t &pdf = response.docInfo[0].metadata["pdf"].values[0];
      if (!pdf.empty()) {
	disp.setmacro ("pagepdf", "Global", pdf);
      }
    }
  }
}

// overridden to format the querystring when highlighting text
void ptmxdocaction::output_document (const text_t &OID, cgiargsclass &args, 
				     recptproto *collectproto, displayclass &disp, 
				     outconvertclass &outconvert, ostream &textout, 
				     ostream &logout) {
  FilterResponse_t inforesponse;
  FilterResponse_t queryresponse;
  text_tset metadata;
  bool getParents = false;
  bool highlight = false;
  int wanttext = 0;
  int arg_gt = args.getintarg("gt");
  text_t &collection = args["c"];
    
  // if we have a query string and highlighting is turned on we need
  // to redo the query to get the terms for highlighting
  if (args.getintarg("hl")) {
    FilterRequest_t request;
    comerror_t err;
    request.filterResultOptions = FRmatchTerms;
    text_t formattedstring = args["q"];
    format_querystring (formattedstring, args.getintarg("b"));

    // ------ for ptmx collection

    if (!args["c1"].empty() && args["c1"] != "Any") {
      //    formattedstring = "[" + args["c1"] + "]:CL";
      formattedstring = args["c1"];
    }
    if (!args["c2"].empty() && args["c2"] != "Any") {
      if (!formattedstring.empty()) formattedstring += " & ";
      //    formattedstring += "[" + args["c2"] + "]:CA";
      formattedstring += args["c2"];
    }
    if (!args["c3"].empty() && args["c3"] != "Any") {
      if (!formattedstring.empty()) formattedstring += " & ";
      //    formattedstring += "[" + args["c3"] + "]:CS";
      formattedstring += args["c3"];
    }
    if (!args["q"].empty()) {
      if (!formattedstring.empty()) formattedstring += " & ";
      formattedstring += args["q"];
    }

    void insert_equiv_terms(text_t&); // declaration
    insert_equiv_terms(formattedstring);
    
    // --------------

    if (!formattedstring.empty()) {

      set_queryfilter_options (request, formattedstring, args);
      collectproto->filter (args["c"], request, queryresponse, err, logout);
      if (err != noError) {
	outconvertclass text_t2ascii;
	logout << text_t2ascii
	       << "documentaction::output_document: call to QueryFilter failed "
	       << "for " << args["c"] << " collection (" << get_comerror_string (err) << ")\n";
	highlight = false;
      } else {
	highlight = true;
      }
    }
  }


  format_t *formatlistptr = new format_t();
  parse_formatstring (formatinfo.DocumentText, formatlistptr, metadata, getParents);

  metadata.insert ("hastxt");
  metadata.insert ("haschildren");

  if (formatinfo.DocumentText == "[Text]")
    wanttext = 1;
  else {
    char *docformat = formatinfo.DocumentText.getcstr();
    if (strstr (docformat, "[Text]") != NULL)
      wanttext = 2;
    delete docformat;
  }
    
  if (get_info (OID, collection, metadata, getParents, collectproto, inforesponse, logout)) {
    int hastxt = inforesponse.docInfo[0].metadata["hastxt"].values[0].getint();
    int haschildren = inforesponse.docInfo[0].metadata["haschildren"].values[0].getint();
    
    if (arg_gt == 0) {
      output_text (inforesponse.docInfo[0], formatlistptr, queryresponse.termInfo, 
		   OID, highlight, hastxt, wanttext, collection, collectproto, 
		   disp, outconvert, textout, logout);


    } else {

      ResultDocInfo_t thisdocinfo = inforesponse.docInfo[0];
	
      // text is to be expanded
      text_t exOID = OID;
      if (haschildren != 1) exOID = get_parent (OID);
      if (exOID.empty()) exOID = OID;
	
      // if we're not in a document (i.e. we're in a top level classification)
      // we need to pass "is_classify = true" to get_contents so that it 
      // doesn't recurse all the way through each document in the classification
      bool is_classify = false;
      if (args["d"].empty()) is_classify = true;

      get_contents (exOID, is_classify, metadata, collection, 
		    collectproto, inforesponse, logout);
	
      ResultDocInfo_tarray::iterator sechere = inforesponse.docInfo.begin();
      ResultDocInfo_tarray::iterator secend = inforesponse.docInfo.end();

      if (arg_gt == 1) {
	// check if there are more than 10 sections containing text to be expanded - 
	// if there are output warning message - this isn't a great way to do this
	// since the sections may be very large or very small - one day I'll fix it 
	// -- Stefan.
	int seccount = 0;
	while (sechere != secend) {
	  int shastxt = (*sechere).metadata["hastxt"].values[0].getint();
	  if (shastxt == 1) seccount ++;
	  if (seccount > 10) break;
	  sechere ++;
	}
	if (seccount > 10) {
	  // more than 10 sections so output warning message and text 
	  // for current section only
	  textout << outconvert << disp << "_document:textltwarning_";

	  output_text (thisdocinfo, formatlistptr, queryresponse.termInfo, 
		       OID, highlight, hastxt, wanttext, collection, 
		       collectproto, disp, outconvert, textout, logout);
	  
	}
	else arg_gt = 2;
      }

      if (arg_gt == 2) {
	// get the text for each section
	sechere = inforesponse.docInfo.begin();
	int count = 0;
	while (sechere != secend) {
	  textout << outconvert << disp << "\n<p><a name=" << (*sechere).OID << "></a>\n";

	  int shastxt = (*sechere).metadata["hastxt"].values[0].getint();

	  output_text (*sechere, formatlistptr, queryresponse.termInfo, 
		       (*sechere).OID, highlight, shastxt, wanttext, collection, 
		       collectproto, disp, outconvert, textout, logout);
	  count ++;
	  sechere ++;
	}
      }
    }
  }
  delete formatlistptr;
}
